@@ -1,40 +1,32 @@
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
-#include "dictmanagedlg.h"
-#include "stardict.h"
-#include "string.h"
+
+#include <cstring>
 #include <sys/stat.h>
+#include <glib/gi18n.h>
+#include <algorithm>
 
 #ifdef _WIN32
 #  include <gdk/gdkwin32.h>
-#  include "win32/intl.h"
 #endif
 
+#include "conf.h"
+#include "lib.h"
+#include "utils.h"
+#include "file.hpp"
 
-DictManageDlg::DictManageDlg()
+#include "dictmanagedlg.h"
+
+DictManageDlg::DictManageDlg(GtkWindow *pw, GdkPixbuf *di,  GdkPixbuf *tdi) :
+	parent_win(pw), dicts_icon(di), tree_dicts_icon(tdi), window(NULL)
 {
-	window = NULL;
 }
 
 void DictManageDlg::response_handler (GtkDialog *dialog, gint res_id, DictManageDlg *oDictManageDlg)
 {
-	switch (res_id) {
-		case GTK_RESPONSE_HELP:
-		{
-#ifdef _WIN32
-			//gchar *filename = g_strdup_printf(_("file:///%s/help/C/stardict.html#stardict-dictmanage"), stardict_data_dir);
-			gchar *filename = g_strdup_printf(_("file:///%s/help/C/stardict.html"), stardict_data_dir);
-			ShellExecute((HWND)(GDK_WINDOW_HWND(gpAppFrame->oAppCore.window->window)), "OPEN", filename, NULL, NULL, SW_SHOWNORMAL);
-			g_free(filename);
-#else
-			gnome_help_display ("stardict.xml", "stardict-dictmanage", NULL);
-#endif
-			break;
-		}
-		default:
-			oDictManageDlg->Close();
-	}
+	if (res_id==GTK_RESPONSE_HELP)
+    show_help("stardict-dictmanage");
 }
 
 void DictManageDlg::on_wazard_button_toggled(GtkToggleButton *button, DictManageDlg *oDictManageDlg)
@@ -49,255 +41,67 @@ void DictManageDlg::on_appendix_button_toggled(GtkToggleButton *button, DictMana
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(oDictManageDlg->notebook), 1);
 }
 
-gboolean DictManageDlg::get_dict_info(gboolean istreedict, const gchar *ifofilename, glong *wordcount, gchar **bookname, gchar **author, gchar **email,
-				gchar **website, gchar **date, gchar **description)
-{
-	struct stat stats;		
-	if (stat (ifofilename, &stats) == -1) {
-		return false;
-	}
-		
-	FILE *file;
-	if (!(file = fopen (ifofilename, "rb"))) {
-		return false;
-	}
-	gchar *buffer = (gchar *)g_malloc (stats.st_size + 1);
-	fread (buffer, 1, stats.st_size, file);
-	buffer[stats.st_size] = '\0';
-	fclose (file);
-	
-	if (!(istreedict?g_str_has_prefix(buffer, "StarDict's treedict ifo file\nversion=2.4.2\n"):g_str_has_prefix(buffer, "StarDict's dict ifo file\nversion=2.4.2\n"))) {
-		g_free(buffer);
-		return false;
-	}
-
-	gchar *p1,*p2,*p3;
-	if (istreedict)
-		p1 = buffer + sizeof("StarDict's treedict ifo file\nversion=2.4.2\n") -1 -1;
-	else
-		p1 = buffer + sizeof("StarDict's dict ifo file\nversion=2.4.2\n") -1 -1;
-
-	p2 = strstr(p1,"\nwordcount=");
-	if (p2) {
-		p3 = strchr(p2+ sizeof("\nwordcount=")-1,'\n');
-		gchar *tmpstr = (gchar *)g_memdup(p2+sizeof("\nwordcount=")-1, p3-(p2+sizeof("\nwordcount=")-1)+1);
-		tmpstr[p3-(p2+sizeof("\nwordcount=")-1)] = '\0';
-		*wordcount = atol(tmpstr);
-		g_free(tmpstr);
-	}
-	else {
-		*wordcount = 0;
-	}
-	
-	p2 = strstr(p1,"\nbookname=");
-	if (p2) {
-		p2 = p2 + sizeof("\nbookname=") -1;
-		p3 = strchr(p2, '\n');
-		*bookname = g_strndup(p2, p3-p2);
-	}
-	else {
-		*bookname = NULL;
-	}
-
-	p2 = strstr(p1,"\nauthor=");
-	if (p2) {
-		p2 = p2 + sizeof("\nauthor=") -1;
-		p3 = strchr(p2, '\n');
-		*author = g_strndup(p2, p3-p2);
-	}
-	else {
-		*author = NULL;
-	}
-
-	p2 = strstr(p1,"\nemail=");
-	if (p2) {
-		p2 = p2 + sizeof("\nemail=") -1;
-		p3 = strchr(p2, '\n');
-		*email = g_strndup(p2, p3-p2);
-	}
-	else {
-		*email = NULL;
-	}
-
-	p2 = strstr(p1,"\nwebsite=");
-	if (p2) {
-		p2 = p2 + sizeof("\nwebsite=") -1;
-		p3 = strchr(p2, '\n');
-		*website = g_strndup(p2, p3-p2);
-	}
-	else {
-		*website = NULL;
-	}
-
-	p2 = strstr(p1,"\ndate=");
-	if (p2) {
-		p2 = p2 + sizeof("\ndate=") -1;
-		p3 = strchr(p2, '\n');
-		*date = g_strndup(p2, p3-p2);
-	}
-	else {
-		*date = NULL;
-	}								
-
-	p2 = strstr(p1,"\ndescription=");
-	if (p2) {
-		p2 = p2 + sizeof("\ndescription=") -1;
-		p3 = strchr(p2, '\n');
-		*description = g_strndup(p2, p3-p2);
-	}
-	else {
-		*description = NULL;
-	}
-
-	g_free(buffer);
-	return true;				
-}
-
-void DictManageDlg::load_dir(gboolean istreedict, gchar *dirname, GSList *order_list, GSList *disable_list, GtkListStore *model)
-{
-	GDir *dir = g_dir_open(dirname, 0, NULL);	
-	if (dir)
-	{
-		GtkTreeIter iter;
-		const gchar *filename;	
-		gchar fullfilename[256];
-		gboolean loaded;
-		GSList *tmplist1,*tmplist2;
-		gboolean disabled;
-		glong wordcount;
-		gchar *bookname, *author, *email, *website, *description, *date;
-		while ((filename = g_dir_read_name(dir))!=NULL) {
-			sprintf(fullfilename, "%s/%s", dirname, filename);
-			if (g_file_test(fullfilename, G_FILE_TEST_IS_DIR)) {
-				load_dir(istreedict, fullfilename, order_list, disable_list, model);
-			}
-			else if (g_str_has_suffix(filename,".ifo")) {
-				tmplist1 = order_list;
-				loaded = false;
-				while (tmplist1) {
-					if (strcmp((gchar *)(tmplist1->data), fullfilename) == 0) {
-						loaded = true;
-						break;
-					}
-					tmplist1 = g_slist_next(tmplist1);
-				}
-				if (loaded)
-					continue;
-				if (get_dict_info(istreedict, fullfilename, &wordcount, &bookname, &author, &email, &website, &description, &date)) {
-					tmplist2 = disable_list;
-					disabled = false;
-					while (tmplist2) {
-						if (strcmp((gchar *)(tmplist2->data), fullfilename) == 0) {
-							disabled = true;
-							break;
-						}
-						tmplist2 = g_slist_next(tmplist2);
-					}
-					gtk_list_store_append (model, &iter);				
-					gtk_list_store_set (model, &iter, 0, !disabled, 1, bookname, 2, wordcount, 3, author, 4, email, 5, website, 6, date, 7, description, 8, fullfilename, 9, true, -1);
-					g_free(bookname);
-					g_free(author);
-					g_free(email);
-					g_free(website);
-					g_free(description);
-					g_free(date);
-				}
-			}
+class GetInfo {
+public:
+	GetInfo(GtkListStore *model_, bool istreedict_):
+		model(model_), istreedict(istreedict_) {}
+	void operator()(const std::string& url, bool disable) {
+		DictInfo dictinfo;
+		if (dictinfo.load_from_ifo_file(url.c_str(), istreedict)) {
+			GtkTreeIter iter;
+			gtk_list_store_append(model, &iter);
+			gtk_list_store_set(model, &iter, 
+												 0, !disable, 
+												 1, dictinfo.bookname.c_str(), 
+												 2, dictinfo.wordcount, 
+												 3, dictinfo.author.c_str(), 
+												 4, dictinfo.email.c_str(), 
+												 5, dictinfo.website.c_str(),
+												 6, dictinfo.description.c_str(), 
+												 7, dictinfo.date.c_str(), 
+												 8, url.c_str(), 
+												 9, TRUE, //for editable.
+												 -1);
 		}
-		g_dir_close(dir);
 	}
-}
-
-GtkTreeModel* DictManageDlg::create_dict_tree_model (gboolean istreedict)
-{
+private:
 	GtkListStore *model;
+	bool istreedict;
+};
 
-	model = gtk_list_store_new (10, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_LONG, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+static GtkTreeModel* __create_dict_tree_model(const std::list<std::string> &dirs,
+																							const std::list<std::string>& order_list,
+																							const std::list<std::string>& disable_list, bool istreedict)
+{
+	GtkListStore *model = 
+		gtk_list_store_new(10, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_LONG, 
+											 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
+											 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
+											 G_TYPE_BOOLEAN);
+	for_each_file(dirs, ".ifo", order_list, disable_list, 
+								GetInfo(model, istreedict));
+	
+	return GTK_TREE_MODEL(model);
+}
 
-	GSList *order_list, *disable_list;
+GtkTreeModel* DictManageDlg::create_dict_tree_model(bool istreedict)
+{
 	if (istreedict) {
-#ifdef _WIN32
-		rw_cfg_read_strlist (usercfgfile, "manage_dictionaries", "treedict_order_list", &order_list);
-		rw_cfg_read_strlist (usercfgfile, "manage_dictionaries", "treedict_disable_list", &disable_list);
-#else
-		gpAppFrame->oAppConf.read_list("/apps/stardict/manage_dictionaries/treedict_order_list", GCONF_VALUE_STRING, &order_list);
-		gpAppFrame->oAppConf.read_list("/apps/stardict/manage_dictionaries/treedict_disable_list", GCONF_VALUE_STRING, &disable_list);
-#endif
-	}
-	else {
-#ifdef _WIN32
-		rw_cfg_read_strlist (usercfgfile, "manage_dictionaries", "dict_order_list", &order_list);
-		rw_cfg_read_strlist (usercfgfile, "manage_dictionaries", "dict_disable_list", &disable_list);
-#else
-		gpAppFrame->oAppConf.read_list("/apps/stardict/manage_dictionaries/dict_order_list", GCONF_VALUE_STRING, &order_list);
-		gpAppFrame->oAppConf.read_list("/apps/stardict/manage_dictionaries/dict_disable_list", GCONF_VALUE_STRING, &disable_list);
-#endif
-	}
+		return __create_dict_tree_model(
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/treedict_dirs_list"),
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/treedict_order_list"),
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/treedict_disable_list"),
+																		istreedict
+																		);
+	} else {
+		return __create_dict_tree_model(
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/dict_dirs_list"),
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/dict_order_list"),
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/dict_disable_list"),
+																		istreedict
+																		);
+	}		
 	
-	glong wordcount;
-	gchar *bookname, *author, *email, *website, *description, *date;
-	
-	GtkTreeIter iter;
-	
-	gchar *ifofilename;
-	GSList *tmplist1,*tmplist2;
-	gboolean disabled;
-	tmplist1 = order_list;	
-	while (tmplist1) {
-		ifofilename = (gchar *)(tmplist1->data);
-		tmplist1 = g_slist_next(tmplist1);
-		
-		if (g_file_test(ifofilename, G_FILE_TEST_EXISTS)) {
-			if (get_dict_info(istreedict, ifofilename, &wordcount, &bookname, &author, &email, &website, &description, &date)) {
-				tmplist2 = disable_list;
-				disabled = false;
-				while (tmplist2) {
-					if (strcmp((gchar *)(tmplist2->data), ifofilename) == 0) {
-						disabled = true;
-						break;
-					}
-					tmplist2 = g_slist_next(tmplist2);
-				}
-				gtk_list_store_append (model, &iter);				
-				gtk_list_store_set (model, &iter, 0, !disabled, 1, bookname, 2, wordcount, 3, author, 4, email, 5, website, 6, date, 7, description, 8, ifofilename, 9, true, -1);
-				g_free(bookname);
-				g_free(author);
-				g_free(email);
-				g_free(website);
-				g_free(description);
-				g_free(date);
-			}
-		}
-	}
-
-#ifdef _WIN32
-	gchar *filename; 	
-	if (istreedict)
-		filename = g_build_filename(stardict_data_dir, "treedict", NULL);		
-	else
-		filename = g_build_filename(stardict_data_dir, "dic", NULL);
-	load_dir(istreedict, filename, order_list, disable_list, model);
-	g_free(filename);
-#else
-	gchar home_dir[256];	
-	if (istreedict)
-		sprintf(home_dir, "%s/.stardict/treedict", g_get_home_dir());
-	else
-		sprintf(home_dir, "%s/.stardict/dic", g_get_home_dir());
-	load_dir(istreedict, home_dir, order_list, disable_list, model);
-	if (istreedict)
-		load_dir(istreedict, STARDICT_DATA_DIR "/treedict", order_list, disable_list, model);
-	else
-		load_dir(istreedict, STARDICT_DATA_DIR "/dic", order_list, disable_list, model);
-#endif
-
-	g_slist_foreach (order_list, (GFunc)g_free, NULL);
-	g_slist_free(order_list);
-	
-	g_slist_foreach (disable_list, (GFunc)g_free, NULL);
-	g_slist_free(disable_list);
-		
-	return GTK_TREE_MODEL (model);	
 }
 
 void DictManageDlg::on_dict_enable_toggled (GtkCellRendererToggle *cell, gchar *path_str, DictManageDlg *oDictManageDlg)
@@ -318,25 +122,19 @@ void DictManageDlg::on_dict_enable_toggled (GtkCellRendererToggle *cell, gchar *
 
 	gboolean have_iter;
 	gchar *filename;
-	GSList *disable_list = NULL;
+	std::list<std::string> disable_list;
 	
 	have_iter = gtk_tree_model_get_iter_first(model, &iter);
 	while (have_iter) {
 		gtk_tree_model_get (model, &iter, 0, &enable, -1);
 		if (!enable) {
 			gtk_tree_model_get (model, &iter, 8, &filename, -1);
-			disable_list = g_slist_append(disable_list, filename);
+			disable_list.push_back(filename);
+			g_free(filename);
 		}
 		have_iter = gtk_tree_model_iter_next(model, &iter);
 	}
-#ifdef _WIN32
-	rw_cfg_write_strlist (usercfgfile, "manage_dictionaries", "dict_disable_list", disable_list);
-#else
-	gpAppFrame->oAppConf.write_list("/apps/stardict/manage_dictionaries/dict_disable_list", GCONF_VALUE_STRING, disable_list);
-#endif
-
-	g_slist_foreach (disable_list, (GFunc)g_free, NULL);
-	g_slist_free(disable_list);
+	conf->set_strlist("/apps/stardict/manage_dictionaries/dict_disable_list", disable_list);
 }
 
 void DictManageDlg::on_treedict_enable_toggled (GtkCellRendererToggle *cell, gchar *path_str, DictManageDlg *oDictManageDlg)
@@ -357,25 +155,19 @@ void DictManageDlg::on_treedict_enable_toggled (GtkCellRendererToggle *cell, gch
 
 	gboolean have_iter;
 	gchar *filename;
-	GSList *disable_list = NULL;
+	std::list<std::string> disable_list;
 	
 	have_iter = gtk_tree_model_get_iter_first(model, &iter);
 	while (have_iter) {
 		gtk_tree_model_get (model, &iter, 0, &enable, -1);
 		if (!enable) {
 			gtk_tree_model_get (model, &iter, 8, &filename, -1);
-			disable_list = g_slist_append(disable_list, filename);
+			disable_list.push_back(filename);
+			g_free(filename);
 		}
 		have_iter = gtk_tree_model_iter_next(model, &iter);
 	}
-#ifdef _WIN32
-	rw_cfg_write_strlist (usercfgfile, "manage_dictionaries", "treedict_disable_list", disable_list);
-#else
-	gpAppFrame->oAppConf.write_list("/apps/stardict/manage_dictionaries/treedict_disable_list", GCONF_VALUE_STRING, disable_list);
-#endif
-
-	g_slist_foreach (disable_list, (GFunc)g_free, NULL);
-	g_slist_free(disable_list);
+	conf->set_strlist("/apps/stardict/manage_dictionaries/treedict_disable_list", disable_list);
 }
 
 void DictManageDlg::drag_data_get_cb(GtkWidget *widget, GdkDragContext *ctx, GtkSelectionData *data, guint info, guint time, DictManageDlg *oDictManageDlg)
@@ -543,12 +335,12 @@ GtkWidget *DictManageDlg::create_dict_tree(gboolean istreedict)
 	return sw;
 }
 
-void DictManageDlg::write_order_list(gboolean istreedict)
+void DictManageDlg::write_order_list(bool istreedict)
 {
 	GtkTreeIter iter;
 	gboolean have_iter;
 	gchar *filename;
-	GSList *order_list = NULL;
+	std::list<std::string> order_list;
 	GtkTreeModel *now_tree_model;
 	
 	if (istreedict)
@@ -559,27 +351,15 @@ void DictManageDlg::write_order_list(gboolean istreedict)
 	have_iter = gtk_tree_model_get_iter_first(now_tree_model, &iter);
 	while (have_iter) {
 		gtk_tree_model_get (now_tree_model, &iter, 8, &filename, -1);
-		order_list = g_slist_append(order_list, filename);
+		order_list.push_back(filename);
+		g_free(filename);
 		have_iter = gtk_tree_model_iter_next(now_tree_model, &iter);
 	}
 	
-	if (istreedict) {
-#ifdef _WIN32
-		rw_cfg_write_strlist (usercfgfile, "manage_dictionaries", "treedict_order_list", order_list);
-#else
-		gpAppFrame->oAppConf.write_list("/apps/stardict/manage_dictionaries/treedict_order_list", GCONF_VALUE_STRING, order_list);
-#endif
-	}
-	else {
-#ifdef _WIN32
-		rw_cfg_write_strlist (usercfgfile, "manage_dictionaries", "dict_order_list", order_list);
-#else
-		gpAppFrame->oAppConf.write_list("/apps/stardict/manage_dictionaries/dict_order_list", GCONF_VALUE_STRING, order_list);
-#endif
-	}
-
-	g_slist_foreach (order_list, (GFunc)g_free, NULL);
-	g_slist_free(order_list);
+	if (istreedict)
+	  conf->set_strlist("/apps/stardict/manage_dictionaries/treedict_order_list", order_list);
+	else
+	  conf->set_strlist("/apps/stardict/manage_dictionaries/dict_order_list", order_list);
 }
 
 void DictManageDlg::on_move_top_button_clicked(GtkWidget *widget, DictManageDlg *oDictManageDlg)
@@ -699,7 +479,11 @@ void DictManageDlg::on_move_down_button_clicked(GtkWidget *widget, DictManageDlg
 GtkWidget *DictManageDlg::create_buttons()
 {
 	GtkWidget *vbox;
+#ifdef CONFIG_GPE
+	vbox = gtk_vbox_new(false,2);
+#else
 	vbox = gtk_vbox_new(false,6);
+#endif
 	GtkWidget *button;
 	button = gtk_button_new_from_stock(GTK_STOCK_GOTO_TOP);
 	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
@@ -722,91 +506,96 @@ GtkWidget *DictManageDlg::create_buttons()
 
 void DictManageDlg::Show()
 {
-	if (window) {
-		gtk_window_present(GTK_WINDOW(window));
-		return;
-	}	
+	if (!window) {	
+		window = gtk_dialog_new();
+		gtk_window_set_transient_for(GTK_WINDOW(window), parent_win);
+
+		gtk_dialog_add_button(GTK_DIALOG(window),
+													GTK_STOCK_HELP,
+													GTK_RESPONSE_HELP);
 	
-	window = gtk_dialog_new();
-	gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (gpAppFrame->oAppCore.window));
+		gtk_dialog_add_button (GTK_DIALOG (window),
+													 GTK_STOCK_CLOSE,
+													 GTK_RESPONSE_CLOSE);
+		gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_CLOSE);
+		g_signal_connect(G_OBJECT(window), "response",
+										 G_CALLBACK(response_handler), this);
+		
+		GtkWidget *vbox;
+#ifdef CONFIG_GPE
+		vbox = gtk_vbox_new (FALSE, 2);
+		gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);
+#else
+		vbox = gtk_vbox_new (FALSE, 6);
+		gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
+#endif
+		
+		GtkWidget *hbox = gtk_hbox_new(false, 3);
+		gtk_box_pack_start(GTK_BOX(vbox),hbox, false, false, 0);
+		
+		wazard_button = gtk_radio_button_new(NULL);
+		GTK_WIDGET_UNSET_FLAGS (wazard_button, GTK_CAN_FOCUS);
+		gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(wazard_button), false);
+		gtk_box_pack_start (GTK_BOX (hbox), wazard_button, false, false, 0);	
+		GtkWidget *hbox1 = gtk_hbox_new(false, 2);
+		gtk_container_add (GTK_CONTAINER (wazard_button), hbox1);
+		GtkWidget *image = gtk_image_new_from_pixbuf(dicts_icon);
+		gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
+		GtkWidget *label = gtk_label_new_with_mnemonic(_("D_ictionaries"));
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label), wazard_button);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wazard_button), true);
+		g_signal_connect(G_OBJECT(wazard_button),"toggled", G_CALLBACK(on_wazard_button_toggled), this);
+		
+		GtkWidget *appendix_button = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(wazard_button));
+		GTK_WIDGET_UNSET_FLAGS (appendix_button, GTK_CAN_FOCUS);
+		gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(appendix_button), false);
+		gtk_box_pack_start (GTK_BOX (hbox), appendix_button, false, false, 0);	
+		hbox1 = gtk_hbox_new(false, 2);
+		gtk_container_add (GTK_CONTAINER (appendix_button), hbox1);
+		image = gtk_image_new_from_pixbuf(tree_dicts_icon);
+		gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
+		label = gtk_label_new_with_mnemonic(_("T_ree dictionaries"));
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label), appendix_button);		
+		g_signal_connect(G_OBJECT(appendix_button),"toggled", G_CALLBACK(on_appendix_button_toggled), this);
+		
+#ifdef CONFIG_GPE
+		hbox = gtk_hbox_new (FALSE, 2);
+#else
+		hbox = gtk_hbox_new (FALSE, 18);
+#endif
+		gtk_box_pack_start (GTK_BOX (vbox), hbox, true, true, 0);
+		
+		notebook = gtk_notebook_new();
+		gtk_box_pack_start(GTK_BOX(hbox),notebook, true, true, 0);
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), false);
+		
+		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), create_dict_tree(false), NULL);
+		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), create_dict_tree(true), NULL);
+		
 
-	gtk_dialog_add_button (GTK_DIALOG (window),
-                             	GTK_STOCK_HELP,
-                             	GTK_RESPONSE_HELP);
+		GtkWidget *buttons = create_buttons ();
+		gtk_box_pack_start (GTK_BOX (hbox), buttons, false, false, 0);		
+		
+		label = gtk_label_new_with_mnemonic (_("These settings will take effect the next time you run StarDict."));
+		gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+		g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
+		gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+		
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), vbox,
+												true, true, 0);
+		
+		
+		gtk_widget_show_all(GTK_DIALOG (window)->vbox);	
 	
-	gtk_dialog_add_button (GTK_DIALOG (window),
-                             	GTK_STOCK_CLOSE,
-                             	GTK_RESPONSE_CLOSE);
-	gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_CLOSE);
-	g_signal_connect(G_OBJECT (window), "response",
-			 G_CALLBACK (response_handler), this);
-
-	GtkWidget *vbox;
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
-	
-	GtkWidget *hbox = gtk_hbox_new(false, 3);
-	gtk_box_pack_start(GTK_BOX(vbox),hbox, false, false, 0);
-	
-	wazard_button = gtk_radio_button_new(NULL);
-	GTK_WIDGET_UNSET_FLAGS (wazard_button, GTK_CAN_FOCUS);
-	gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(wazard_button), false);
-	gtk_box_pack_start (GTK_BOX (hbox), wazard_button, false, false, 0);	
-	GtkWidget *hbox1 = gtk_hbox_new(false, 2);
-	gtk_container_add (GTK_CONTAINER (wazard_button), hbox1);
-	GtkWidget *image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.stardict.index_wazard.p[0]);
-	gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
-	GtkWidget *label = gtk_label_new_with_mnemonic(_("D_ictionaries"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), wazard_button);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wazard_button), true);
-	g_signal_connect(G_OBJECT(wazard_button),"toggled", G_CALLBACK(on_wazard_button_toggled), this);
-
-	GtkWidget *appendix_button = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(wazard_button));
-	GTK_WIDGET_UNSET_FLAGS (appendix_button, GTK_CAN_FOCUS);
-	gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(appendix_button), false);
-	gtk_box_pack_start (GTK_BOX (hbox), appendix_button, false, false, 0);	
-	hbox1 = gtk_hbox_new(false, 2);
-	gtk_container_add (GTK_CONTAINER (appendix_button), hbox1);
-	image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.stardict.index_appendix.p[0]);
-	gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
-	label = gtk_label_new_with_mnemonic(_("T_ree dictionaries"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), appendix_button);		
-	g_signal_connect(G_OBJECT(appendix_button),"toggled", G_CALLBACK(on_appendix_button_toggled), this);
-
-	hbox = gtk_hbox_new (FALSE, 18);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, true, true, 0);
-
-	notebook = gtk_notebook_new();
-	gtk_box_pack_start(GTK_BOX(hbox),notebook, true, true, 0);
-	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), false);
-
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), create_dict_tree(false), NULL);
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), create_dict_tree(true), NULL);
-	
-
-	GtkWidget *buttons;
-	buttons = create_buttons ();
-	gtk_box_pack_start (GTK_BOX (hbox), buttons, false, false, 0);		
-
-	label = gtk_label_new_with_mnemonic (_("These settings will take effect the next time you run StarDict."));
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), vbox,
-			true, true, 0);
-
-	
-	gtk_widget_show_all (GTK_DIALOG (window)->vbox);	
-	
-	gtk_window_set_title (GTK_WINDOW (window), _("Manage Dictionaries"));
-	//gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
-	gtk_widget_show(window);
-
+		gtk_window_set_title(GTK_WINDOW (window), _("Manage Dictionaries"));	
+	}
+	while (gtk_dialog_run(GTK_DIALOG(window))==GTK_RESPONSE_HELP)
+    ;
+	gtk_widget_hide(GTK_WIDGET(window));
 }
 
 void DictManageDlg::Close()
