@@ -1,3 +1,22 @@
+/* 
+ * This file part of StarDict - A international dictionary for GNOME.
+ * http://stardict.sourceforge.net
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -5,8 +24,8 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "stardict.h"
 #include "conf.h"
+#include "utils.h"
 
 #include "splash.h"
 
@@ -22,7 +41,8 @@ void show_splash_screen()
 {		
 	gtk_window_set_auto_startup_notification(false);
 
-	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file((gStarDictDataDir+G_DIR_SEPARATOR+"pixmaps"+G_DIR_SEPARATOR+"splash.png").c_str(), NULL);
+	GdkPixbuf *pixbuf = 
+		load_image_from_file(gStarDictDataDir+ G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "splash.png");
 
 	gint w = gdk_pixbuf_get_width(pixbuf);
 	gint h = gdk_pixbuf_get_height(pixbuf);
