@@ -1,9 +1,13 @@
 #ifndef __STAR_DICT_H__
 #define __STAR_DICT_H__
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <gtk/gtk.h>
 
-#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
+#ifdef CONFIG_GNOME
 # include <bonobo/bonobo-object.h>
 # include <libgnome/libgnome.h>
 #endif
@@ -12,20 +16,23 @@
 class AppCore;
 class AppFrame;
 
-#include "skin.h"
+#include "conf.h"
 #include "lib.h"
+#include "skin.h"
 #include "mainwin.h"
 #ifdef _WIN32
-#  include "win32/clipboard.h"	
 #  include "win32/systray.h"
+#  include "win32/clipboard.h"
+#  include "win32/mouseover.h"
+#  include "win32/hotkey.h"
 #else
 #  include "docklet.h"
 #endif
 #include "floatwin.h"
 #include "selection.h"
-#include "prefsdlg.h"
 #include "dictmanagedlg.h"
 #include "readword.h"
+#include "iskeyspressed.hpp"
 
 extern AppFrame * gpAppFrame;
 
@@ -37,18 +44,13 @@ const int MAX_FLOAT_WINDOW_FUZZY_MATCH_ITEM=5;
 const int LIST_WIN_ROW_NUM = 30; //how many words show in the list win.
 
 
-class AppCore
-{
+class AppCore {
 private:
-	AppFrame* poAppFrame;
-
-	PrefsDlg *prefs_dlg;
 	DictManageDlg *dict_manage_dlg;
 
 	static int MatchWordCompare(const void * s1, const void * s2);
 	static gboolean on_delete_event(GtkWidget * window, GdkEvent *event , AppCore *oAppCore);
 	static gboolean on_window_state_event(GtkWidget * window, GdkEventWindowState *event , AppCore *oAppCore);
-	static gboolean on_window_visibility_notify(GtkWidget * window, GdkEventVisibility *event , AppCore *oAppCore);
 	static gboolean vKeyPressReleaseCallback(GtkWidget * window, GdkEventKey *event , AppCore *oAppCore);
 public:
 	glong *iCurrentIndex;
@@ -62,28 +64,34 @@ public:
 	Selection oSelection;
 #ifdef _WIN32
 	Clipboard oClipboard;
+	Mouseover oMouseover;
+	Hotkey oHotkey;
 #endif
 	FloatWin oFloatWin;
 	DockLet oDockLet;
 
 	Libs oLibs;
 	TreeDicts oTreeDicts;
+	std::auto_ptr<hotkeys> unlock_keys;
+	AppSkin oAppSkin;
 
-	AppCore(AppFrame* pAppFrame);
+	AppCore();
 	~AppCore();
 	void Create(gchar *queryword);
-    void End();
+	void End();
 	void Query(const gchar *word);
 	bool SimpleLookupToFloat(const gchar* sWord, bool bShowIfNotFound);
+#ifdef _WIN32
+	bool SmartLookupToFloat(const gchar* sWord, int BeginPos, bool bShowIfNotFound);
+#endif
 	bool SimpleLookupToTextWin(const gchar* sWord,glong* piIndex, bool piIndexValid = false, bool bTryMoreIfNotFound = false);
+	void LookupDataToMainWin(const gchar *sWord);
 	void LookupWithFuzzyToMainWin(const gchar* word);
 	void LookupWithFuzzyToFloatWin(const gchar * word);
 	void LookupWithRuleToMainWin(const gchar* word);
-	void ShowDataToTextWin(gchar ** ppWord, gchar ** ppWordData,const gchar * sOriginWord);
-	void ShowTreeDictDataToTextWin(glong offset, glong size, gint iTreeDict);
-	void ShowNotFoundToTextWin(const char* sWord,const char* sReason, TextWinQueryResult query_result);	
-	void ShowDataToFloatWin(gchar ** ppWord, gchar ** ppWordData,const gchar * sOriginWord);
-	void ShowDatasToFloatWin(gchar *** pppWord, gchar *** pppWordData, const gchar ** ppOriginWord, gint count, const gchar * sOriginWord);
+	void ShowDataToTextWin(const gchar **ppWord, gchar ** ppWordData,const gchar * sOriginWord);
+	void ShowTreeDictDataToTextWin(guint32 offset, guint32 size, gint iTreeDict);
+	void ShowNotFoundToTextWin(const char* sWord,const char* sReason, TextWinQueryResult query_result);
 	void ShowNotFoundToFloatWin(const char* sWord,const char* sReason, gboolean fuzzy);
 
 	void TopWinEnterWord(const gchar *text);
@@ -95,14 +103,11 @@ public:
 	void PopupDictManageDlg();
 };
 
-class AppFrame {
+class AppFrame : public AppCore {
 public:
-	AppCore oAppCore;
-	AppSkin oAppSkin;
 	ReadWord oReadWord;
 
-
-#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
+#ifdef CONFIG_GNOME
 	BonoboObject *stardict_app_server;
 #endif
 
@@ -110,13 +115,21 @@ public:
 	~AppFrame();
 	void Init(gchar *queryword);
 	void Quit();
+
+private:
+	static void on_main_win_hide_list_changed(const baseconfval*, void *arg);
+	static void on_dict_scan_select_changed(const baseconfval*, void *arg);
+	static void on_floatwin_lock_changed(const baseconfval*, void *arg);
+
+	static void on_floatwin_lock_x_changed(const baseconfval*, void *arg);
+	static void on_floatwin_lock_y_changed(const baseconfval*, void *arg);
+	static void on_scan_modifier_key_changed(const baseconfval*, void *arg);
 };
 
 #ifdef _WIN32
 extern HINSTANCE stardictexe_hInstance;
 #endif
 
-extern gboolean bContainRule(const char* sWord);
 extern gchar* GetPureEnglishAlpha(gchar *str);
 extern gchar* GetHeadWord(gchar *str);
 extern gboolean stardict_on_enter_notify (GtkWidget * widget, GdkEventCrossing * event, gpointer data);
